###################################################################################################
# _metaobjmanager.py
#
# Implementation of class MetaobjManager (see below).
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###################################################################################################

# Imports.
from Globals import HTML, HTMLFile
import string
import urllib
import copy
# Product Imports.
import _globals
import _xmllib


# -------------------------------------------------------------------------------------------------
#  synchronizeMetaObjs
# -------------------------------------------------------------------------------------------------
def synchronizeMetaObjs(self):
  req = {}
  if self.meta_type == 'ZMSCustom':
    if not self.meta_id in self.getMetaobjIds(req):
      #++ print "[%s.synchronizeMetaObjs]: Delete %s(%s)..."%(self.meta_type,self.id,self.meta_id)
      parent = self.getParentNode()
      parent.manage_delObjects([self.id])
    else:
      obj_attrs = {}
      attrs = self.getMetaobj(self.meta_id)['attrs']
      for attr in attrs:
        obj_attrs[attr['id']] = attr
      if _globals.debug: print "[%s.synchronizeMetaObjs]: obj_attrs.keys=%s"%(self.meta_type,str(obj_attrs.keys()))
      ids = self.objectIds()
      for id in ids:
        if id != 'acl_users':
          id_prefix = _globals.id_prefix(id)
          delete = 0
          delete = delete or (id not in obj_attrs.keys() and id_prefix not in obj_attrs.keys())
          delete = delete or (id in obj_attrs.keys() and obj_attrs[id]['type'] not in self.dGlobalAttrs.keys())
          delete = delete or (id_prefix in obj_attrs.keys() and obj_attrs[id_prefix]['type'] not in self.dGlobalAttrs.keys())
          if delete:
            if _globals.debug: print "[%s.synchronizeMetaObjs]: Delete %s(%s)..."%(self.meta_type,id,id_prefix)
            self.manage_delObjects([id])
  for child in self.objectValues():
    synchronizeMetaObjs(child)

# -------------------------------------------------------------------------------------------------
#  findMetaObj
# -------------------------------------------------------------------------------------------------
def findMetaObj(self, meta_id):
  if self.meta_type == 'ZMSCustom' and self.meta_id == meta_id:
    return self
  for child in self.objectValues():
    metaobj = findMetaObj(child,meta_id)
    if metaobj is not None:
      return metaobj
  return None

# -------------------------------------------------------------------------------------------------
#  getTemplateId
# -------------------------------------------------------------------------------------------------
def getTemplateId(id, metatype):
  if metatype == 'ZMSDocument':
    return "bodyContentZMSCustom_%s"%id
  if metatype == 'ZMSTeaserElement':
    return "bodyContentTeaserZMSCustom_%s"%id
  else:
    return "ZMSCustom_%s"%id


###################################################################################################
###################################################################################################
###
###   class MetaobjManager
###
###################################################################################################
###################################################################################################
class MetaobjManager: 

    # Management Permissions.
    # -----------------------
    __authorPermissions__ = ()
    __editorPermissions__ = ()
    __administratorPermissions__ = (
		'manage_changeMetaobjs','manage_addMetaobj','manage_customizeMetaobjForm',
		)
    __ac_permissions__=(
		('ZMS Administrator', __administratorPermissions__),
		('ZMS Author', __authorPermissions__),
		('ZMS Editor', __editorPermissions__),
		)


    # Management Interface to customize Meta-Objects.
    manage_customizeMetaobjForm = HTMLFile('dtml/metaobj/manage_customizeform', globals()) 
    metaobj_input_fields = HTMLFile('dtml/metaobj/input_fields', globals())
    metaobj_input_js = HTMLFile('dtml/metaobj/input_js', globals())


    # ------------------------------------------------------------------------------------------
    # MetaobjManager.recurse_updateVersionMetaObjs: 
    #
    # Update version.
    # ------------------------------------------------------------------------------------------
    def recurse_updateVersionMetaObjs(self, lang, manage_lang, REQUEST):
      message = ""
      #++ print "[MetaobjManager.recurse_updateVersionMetaObjs]: "
      
      # -----------------------------------------------------------
      # 20.02.2002
      # Build #060: Make object-attributes multilingual.
      # -----------------------------------------------------------
      if getattr(self.getDocumentElement(),'build','') < '060':
        obs = self.__get_metaobjs__()
        for id in obs.keys():
          ob = self.__get_metaobj__(id)
          attrs = copy.copy(ob['__obj_attrs__'])
          for attr in attrs:
            datatype = attr['type']
            if string.find(datatype,'ZMS') < 0 and \
               string.find(datatype,'Lang') < 0:
              attr['type'] = 'Lang%s'%string.capitalize(datatype)
              # attrs[ids.index(attr_id)] = attr
          # Set Attributes to Meta-Object.
          ob['__obj_attrs__'] = attrs
          self.__attr_dc_metaobjs__[id] = ob
          self.__attr_dc_metaobjs__ = self.__attr_dc_metaobjs__.copy()

      # -----------------------------------------------------------
      # 17.04.2002
      # Build #071: Meta-Object Type.
      # -----------------------------------------------------------
      if getattr(self.getDocumentElement(),'build','') < '071':
        obs = self.__get_metaobjs__()
        for id in obs.keys():
          ob = self.__get_metaobj__(id)
	  tmplt_ids = self.getDocumentElement().aq_parent.objectIds(['DTML Method'])
	  if 'ZMSCustom_%s'%id in tmplt_ids:
	    ob['type'] = 'ZMSObject'
	  else:
	    ob['type'] = 'ZMSDocument'
          self.__attr_dc_metaobjs__[id] = ob
          self.__attr_dc_metaobjs__ = self.__attr_dc_metaobjs__.copy()

      # -----------------------------------------------------------
      # 20.08.2002
      # Build #094: Meta-Object Attribute Keys.
      # -----------------------------------------------------------
      if getattr(self.getDocumentElement(),'build','') < '094':
        obs = self.__get_metaobjs__()
        for id in obs.keys():
          ob = self.__get_metaobj__(id)
          attrs = []
          for attr in ob['__obj_attrs__']:
            attr_keys = ''
            if attr.has_key('keys'):
              attr_keys = attr['keys']
              attr_keys = string.replace(attr_keys,',','\n')
            attr['keys'] = attr_keys
            attrs.append(attr)
          ob['__obj_attrs__'] = attrs
          self.__attr_dc_metaobjs__[id] = ob
          self.__attr_dc_metaobjs__ = self.__attr_dc_metaobjs__.copy()

      # -----------------------------------------------------------
      # 22.08.2002
      # Build #096: Replace Meta-Object Attribute Type Html by Text.
      # -----------------------------------------------------------
      if getattr(self.getDocumentElement(),'build','') < '096':
        obs = self.__get_metaobjs__()
        for id in obs.keys():
          ob = self.__get_metaobj__(id)
          attrs = []
          for attr in ob['__obj_attrs__']:
            attr_type = attr['type']
            if attr_type == 'LangHtml':
              attr_type = 'LangText'
            attr['type'] = attr_type
            attrs.append(attr)
          ob['__obj_attrs__'] = attrs
          self.__attr_dc_metaobjs__[id] = ob
          self.__attr_dc_metaobjs__ = self.__attr_dc_metaobjs__.copy()
      
      ##### SYNCHRONIZE ####      
      self.synchronizeObjAttrs()
      
      ##### Delete Meta-Object zombies. ####
      synchronizeMetaObjs(self)
      
      return message

    # ------------------------------------------------------------------------------------------
    # MetaobjManager.synchronizeMetaObjs: 
    # ------------------------------------------------------------------------------------------
    def synchronizeMetaObjs(self):
      synchronizeMetaObjs(self)

    # ------------------------------------------------------------------------------------------
    # MetaobjManager.__init_metaobjs__:
    # 
    # Initializes dictionary of Meta-Objects.
    # ------------------------------------------------------------------------------------------
    def __init_metaobjs__(self):
      self.__attr_dc_metaobjs__ = {}

    # ------------------------------------------------------------------------------------------
    # MetaobjManager.__get_metaobjs__:
    # 
    # Returns dictionary of Meta-Objects.
    # ------------------------------------------------------------------------------------------
    def __get_metaobjs__(self):
      ob = self.getDocumentElement()
      if not hasattr(ob,'__attr_dc_metaobjs__'):
        ob.__init_metaobjs__()
      obs = getattr(ob,'__attr_dc_metaobjs__')
      return obs

    # ------------------------------------------------------------------------------------------
    # MetaobjManager.__get_metaobj__:
    # 
    # Returns Meta-Object identified by ID.
    # ------------------------------------------------------------------------------------------
    def __get_metaobj__(self, id): 
      return self.__get_metaobjs__()[id]

    # ------------------------------------------------------------------------------------------
    # MetaobjManager.getMetaobjIds:
    # 
    # Returns list of IDs of Meta-Objects.
    # ------------------------------------------------------------------------------------------
    def getMetaobjIds(self, REQUEST=None):
      obs = self.__get_metaobjs__()
      tmp = []
      for id in obs.keys():
        ob = self.__get_metaobj__(id)
        tmp.append((ob['name'],id))
      tmp.sort()
      ids = []
      for ob in tmp:
        ids.append(ob[1])
      return ids

    # ------------------------------------------------------------------------------------------
    # MetaobjManager.getMetaobj:
    # 
    # Returns Meta-Object identified by ID.
    # ------------------------------------------------------------------------------------------
    def getMetaobj(self, id):
      ob = self.__get_metaobj__(id)
      # Retrieve properties.
      name = ob['name']
      type = ob['type']
      attrs = ob['__obj_attrs__']
      tmplt = getTemplateId(id,type)
      # Return dictionary of properties.
      return {'id':id,'name':name,'type':type,'attrs':attrs,'tmplt':tmplt}

    # ------------------------------------------------------------------------------------------
    # MetaobjManager.getMetaobjId:
    # 
    # Returns Meta-Object ID identified by Name.
    # ------------------------------------------------------------------------------------------
    def getMetaobjId(self, name):
      for id in self.getMetaobjIds():
        ob = self.getMetaobj(id)
        if name == ob['name']:
          return id
      return None


    """
    ###############################################################################################
    #
    #   A T T R I B U T E S
    #
    ###############################################################################################
    """

    # ------------------------------------------------------------------------------------------
    # MetaobjManager._initMetaObjAttrs:
    # 
    # Init list of Attributes for Meta-Object specified by ID.
    # ------------------------------------------------------------------------------------------
    def _initMetaObjAttrs(self, id):
      attrs = []
      cp = copy.copy(attrs)
      # Init Attributes.
      ob = self.__get_metaobj__(id)
      ob['__obj_attrs__'] = attrs
      # Set Attributes to Meta-Object.
      self.__attr_dc_metaobjs__[id] = ob
      self.__attr_dc_metaobjs__ = self.__attr_dc_metaobjs__.copy()

    # ------------------------------------------------------------------------------------------
    # MetaobjManager._getMetaObjAttrIds:
    # 
    # Returns list of Attribute-IDs for Meta-Object specified by ID.
    # ------------------------------------------------------------------------------------------
    def _getMetaObjAttrIds(self, id):
      ids = []
      ob = self.__get_metaobj__(id)
      attrs = ob['__obj_attrs__']
      for attr in attrs:
        ids.append(attr['id'])
      return ids

    # ------------------------------------------------------------------------------------------
    # MetaobjManager._setMetaObjAttr:
    # 
    # Set/add Meta-Object Attribute with specified values.
    # ------------------------------------------------------------------------------------------
    def _setMetaObjAttr(self, id, attr_id, attr_name, attr_mandatory=0, attr_repetitive=0, attr_type='LangString', attr_keys=''):
      ob = self.__get_metaobj__(id)
      attrs = copy.copy(ob['__obj_attrs__'])
      # Set Attributes.
      if not attr_type in ['LangString','LangText']:
        attr_keys = ''
      if attr_type[:3] == 'ZMS':
        attr_mandatory = 0
      else:
        attr_repetitive = 0
      attr = {}
      attr['id'] = attr_id
      attr['name'] = attr_name
      attr['mandatory'] = attr_mandatory
      attr['repetitive'] = attr_repetitive
      attr['type'] = attr_type
      attr['keys'] = attr_keys
      ids = self._getMetaObjAttrIds(id)
      if attr_id in ids:
        attrs[ids.index(attr_id)] = attr
      else:
        attrs.append(attr)
      # Set Attributes to Meta-Object.
      ob['__obj_attrs__'] = attrs
      self.__attr_dc_metaobjs__[id] = ob
      self.__attr_dc_metaobjs__ = self.__attr_dc_metaobjs__.copy()

    # ------------------------------------------------------------------------------------------
    # MetaobjManager._moveMetaObjAttr:
    # 
    # Moves Meta-Object Attribute in specified direction (-1=up, +1=down).
    # ------------------------------------------------------------------------------------------
    def _moveMetaObjAttr(self, id, attr_id, direction=0):
      ob = self.__get_metaobj__(id)
      attrs = copy.copy(ob['__obj_attrs__'])
      # Move Attributes.
      ids = self._getMetaObjAttrIds(id)
      i = ids.index(attr_id)
      attr = attrs[i]
      attrs[i] = attrs[i+direction]
      attrs[i+direction] = attr
      # Set Attributes to Meta-Object.
      ob['__obj_attrs__'] = attrs
      self.__attr_dc_metaobjs__[id] = ob
      self.__attr_dc_metaobjs__ = self.__attr_dc_metaobjs__.copy()
      
    # ------------------------------------------------------------------------------------------
    # MetaobjManager._delMetaObjAttr:
    # 
    # Delete Attribute from Meta-Object specified by ID..
    # ------------------------------------------------------------------------------------------
    def _delMetaObjAttr(self, id, attr_id):
        # Delete Attribute.
        ob = self.__get_metaobj__(id)
        cp = []
        attrs = copy.copy(ob['__obj_attrs__'])
        for attr in attrs:
          if attr['id'] != attr_id:
            cp.append(attr)
        ob['__obj_attrs__'] = cp
        # Set Attributes to Meta-Object.
        self.__attr_dc_metaobjs__[id] = ob
        self.__attr_dc_metaobjs__ = self.__attr_dc_metaobjs__.copy()

    ###############################################################################################

    # ------------------------------------------------------------------------------------------
    # MetaobjManager.__set_metaobj__:
    # 
    # Set/add Meta-Object with specified values.
    # ------------------------------------------------------------------------------------------
    def __init_metaobj__(self, id, ob, old_metatype=None, dtml_raw=None):
    
      # Init Meta-Object.
      # -----------------
      self.__get_metaobjs__()
      
      # Template.
      # ---------
      root = self.getDocumentElement().aq_parent
      dtml_id = getTemplateId(id,ob['type'])
      dtml_title = "Template: %s"%ob['name']
      if dtml_raw is None:
        dtml_raw = ""
        dtml_raw = dtml_raw + "<!-- BO %s -->\n"%dtml_id
        dtml_raw = dtml_raw + "<!-- EO %s -->\n"%dtml_id
      # Insert.
      if old_metatype is None or not hasattr(self,dtml_id):
        root.manage_addDTMLMethod(dtml_id,dtml_title,dtml_raw)
      # Rename
      elif old_metatype != ob['type']:
        root.manage_renameObject(getTemplateId(id,old_metatype),dtml_id)
      
      # Set attribute values.
      self.__attr_dc_metaobjs__[id] = ob
      self.__attr_dc_metaobjs__ = self.__attr_dc_metaobjs__.copy()


    # ------------------------------------------------------------------------------------------
    # MetaobjManager.__set_metaobj__:
    # 
    # Set/add Meta-Object with specified values.
    # ------------------------------------------------------------------------------------------
    def __set_metaobj__(self, id, name, metatype, attrs=[]):
      
      # Init Meta-Object.
      # -----------------
      self.__get_metaobjs__()
      
      if id in self.__attr_dc_metaobjs__.keys():
        old_metatype = self.__attr_dc_metaobjs__[id]['type']
      else:
        old_metatype = None
      
      # Set Meta-Object.
      # ----------------
      ob = {}
      ob['name'] = name
      ob['type'] = metatype
      ob['__obj_attrs__'] = copy.copy(attrs)
      self.__init_metaobj__(id,ob,old_metatype)


    # ------------------------------------------------------------------------------------------
    # MetaobjManager.__del_metaobj__:
    # 
    # Delete Meta-Object specified by ID.
    # ------------------------------------------------------------------------------------------
    def __del_metaobj__(self, id):
      
      # Delete from attribute values.
      cp = self.__attr_dc_metaobjs__
      self.__init_metaobjs__()
      metatype = None
      for ob in cp.keys():
        if ob == id:
          metatype = cp[ob]['type']
        else:
          self.__attr_dc_metaobjs__[ob] = cp[ob]
      
      # Delete Template.
      # ----------------
      dtml_id = getTemplateId(id,metatype)
      root = self.getDocumentElement().aq_parent
      root.manage_delObjects(ids=[dtml_id])
      
      # Set attribute values.
      self.__attr_dc_metaobjs__ = self.__attr_dc_metaobjs__.copy()


    ###############################################################################################
    # MetaobjManager.manage_changeMetaobjs:
    #
    # Werte fr Meta-Objects lschen oder ndern.
    ###############################################################################################
    def manage_changeMetaobjs(self, lang, manage_lang, btn='', id='', key='all', REQUEST=None, RESPONSE=None):
        """ MetaobjManager.manage_changeMetaobjs """
        
        message = ""
        
        # Delete.
        # -------
        
        # Obj.
        if key == 'obj' and btn == self.getLangStr('BTN_DELETE',manage_lang):
          metaobj = findMetaObj(self,id)
          if metaobj is None:
            self.__del_metaobj__(id)
            id = ''
            # Delete Object-Instances.
            synchronizeMetaObjs(self)
            message = self.getLangStr('MSG_CHANGED',manage_lang)
          else:
            message = '[ZMS-ERROR]: Constraint violation at <a href="%s/manage_main?lang=%s&manage_lang=%s#_%s">%s</a>'%(metaobj.getParentNode().absolute_url(),lang,manage_lang,metaobj.id,metaobj.absolute_url())
        
        # Attr.
        if key == 'attr' and btn == 'delete':
          attr_id = REQUEST['attr_id']
          self._delMetaObjAttr(id,attr_id)
        
        # Change.
        # -------
        elif key == 'all' and btn == self.getLangStr('BTN_CHANGE',manage_lang):
        
          # Obj.
          id = REQUEST['id']
          name = REQUEST['obj_name']
          metatype = REQUEST['obj_type']
          attrs = ob = self.__get_metaobj__(id)
          attrs = ob['__obj_attrs__']
          self.__set_metaobj__(id,name,metatype,attrs)
          meta_obj = self.getMetaobj(id)
          if hasattr(self,meta_obj['tmplt']):
            ob = getattr(self,meta_obj['tmplt'])
            ob.manage_edit(title='Template: %s'%meta_obj['name'],data=REQUEST.get('obj_dtml',''))
          
          # Attr.
          saved_ids = self._getMetaObjAttrIds(id)
          self._initMetaObjAttrs(id)
          for saved_id in saved_ids:
            attr_id = REQUEST['attr_id_%s'%saved_id]
            attr_name = REQUEST['attr_name_%s'%saved_id]
            attr_mandatory = REQUEST.get('attr_mandatory_%s'%saved_id,0)
            attr_repetitive = REQUEST.get('attr_repetitive_%s'%saved_id,0)
            attr_type = REQUEST['attr_type_%s'%saved_id]
            attr_keys = REQUEST.get('attr_keys_%s'%saved_id,'')
            self._setMetaObjAttr(id,attr_id,attr_name,attr_mandatory,attr_repetitive,attr_type,attr_keys)
          
          message = self.getLangStr('MSG_CHANGED',manage_lang)
        
        # Export.
        # -------
        elif btn == self.getLangStr('BTN_EXPORT',manage_lang):
          id = REQUEST['id']
          ob = self.__get_metaobj__(id)
          root = self.getDocumentElement().aq_parent
          dtml_id = getTemplateId(id,ob['type'])
          dtml_raw = ""
          for method in root.objectValues(['DTML Method']):
            if method.id() == dtml_id:
              dtml_raw = method.raw
          content_type = 'text/xml'
          filename = id + '.xml'
          export = '<?xml version="1.0" encoding="ISO-8859-1"?>'
          export = export + _xmllib.toXml({'key':id,'value':ob,'dtml':dtml_raw})
          RESPONSE.setHeader('Content-Type',content_type)
          RESPONSE.setHeader('Content-Disposition','inline;filename=%s'%filename)
          return export
        
        # Insert.
        # -------
        elif btn == self.getLangStr('BTN_INSERT',manage_lang):
        
          # Obj.
          if key == 'obj':
            id = string.strip(REQUEST['_meta_id'])
            name = string.strip(REQUEST['_meta_name'])
            metatype = string.strip(REQUEST['_meta_type'])
            self.__set_metaobj__(id,name,metatype)
            if metatype == 'ZMSDocument':
              self._setMetaObjAttr(id,'titlealt',self.getLangStr('ATTR_TITLESHORT',manage_lang))
              self._setMetaObjAttr(id,'title',self.getLangStr('ATTR_TITLE',manage_lang))
            elif metatype == 'ZMSTeaserElement':
              self._setMetaObjAttr(id,'titlealt',self.getLangStr('ATTR_TITLESHORT',manage_lang))
          
          # Attr.
          if key == 'attr':
            attr_id = REQUEST['_id']
            attr_name = REQUEST['_name']
            attr_mandatory = REQUEST.get('_mandatory',0)
            attr_repetitive = REQUEST.get('_repetitive',0)
            attr_type = REQUEST.get('_type','')
            attr_keys = REQUEST.get('_keys','')
            self._setMetaObjAttr(id,attr_id,attr_name,attr_mandatory,attr_repetitive,attr_type,attr_keys)
        
        # Import.
        # -------
        elif btn == self.getLangStr('BTN_IMPORT',manage_lang):
          builder = _xmllib.XmlAttrBuilder()
          value = builder.parse(REQUEST['file'])
          id = value['key']
          ob = value['value']
          old_metatype = None
          dtml_raw = value['dtml']
          self.__init_metaobj__(id,ob,old_metatype,dtml_raw)
        
        # Move up.
        # --------
        elif key == 'attr' and btn == 'move_up':
          attr_id = REQUEST['attr_id']
          self._moveMetaObjAttr(id,attr_id,-1)
        
        # Move down.
        # ----------
        elif key == 'attr' and btn == 'move_down':
          attr_id = REQUEST['attr_id']
          self._moveMetaObjAttr(id,attr_id,+1)
        
        ##### SYNCHRONIZE ####      
        self.synchronizeObjAttrs()
        
        # Return with message.
        message = urllib.quote(message)
        return RESPONSE.redirect('manage_customizeMetaobjForm?lang=%s&manage_lang=%s&manage_tabs_message=%s&id=%s'%(lang,manage_lang,message,id))

###################################################################################################
